@@ -527,6 +527,9 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	QObject::connect(m_ui.viewEventsAction,
 		SIGNAL(triggered(bool)),
 		SLOT(viewEvents(bool)));
+	QObject::connect(m_ui.viewNoteNamesAction,
+		SIGNAL(triggered(bool)),
+		SLOT(viewNoteNames(bool)));
 	QObject::connect(m_ui.viewNoteDurationAction,
 		SIGNAL(triggered(bool)),
 		SLOT(viewNoteDuration(bool)));
@@ -677,6 +680,7 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 		m_ui.viewToolbarTimeAction->setChecked(pOptions->bMidiTimeToolbar);
 		m_ui.viewToolbarScaleAction->setChecked(pOptions->bMidiScaleToolbar);
 		m_ui.viewToolbarThumbAction->setChecked(pOptions->bMidiThumbToolbar);
+		m_ui.viewNoteNamesAction->setChecked(pOptions->bMidiNoteNames);
 		m_ui.viewNoteDurationAction->setChecked(pOptions->bMidiNoteDuration);
 		m_ui.viewNoteColorAction->setChecked(pOptions->bMidiNoteColor);
 		m_ui.viewValueColorAction->setChecked(pOptions->bMidiValueColor);
@@ -714,6 +718,7 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 		m_pMidiEditor->setEditModeDraw(pOptions->bMidiEditModeDraw);
 		m_pMidiEditor->setNoteColor(pOptions->bMidiNoteColor);
 		m_pMidiEditor->setValueColor(pOptions->bMidiValueColor);
+		m_pMidiEditor->setNoteNames(pOptions->bMidiNoteNames);
 		m_pMidiEditor->setNoteDuration(pOptions->bMidiNoteDuration);
 		m_pMidiEditor->setSendNotes(pOptions->bMidiPreview);
 		m_pMidiEditor->setSyncView(pOptions->bMidiFollow);
@@ -918,6 +923,7 @@ void qtractorMidiEditorForm::closeEvent ( QCloseEvent *pCloseEvent )
 		pOptions->bMidiEditMode = m_pMidiEditor->isEditMode();
 		pOptions->bMidiEditModeDraw = m_pMidiEditor->isEditModeDraw();
 		pOptions->iMidiDisplayFormat = (m_pMidiEditor->timeScale())->displayFormat();
+		pOptions->bMidiNoteNames = m_ui.viewNoteNamesAction->isChecked();
 		pOptions->bMidiNoteDuration = m_ui.viewNoteDurationAction->isChecked();
 		pOptions->bMidiNoteColor = m_ui.viewNoteColorAction->isChecked();
 		pOptions->bMidiValueColor = m_ui.viewValueColorAction->isChecked();
@@ -1722,6 +1728,14 @@ void qtractorMidiEditorForm::viewValueColor ( bool bOn )
 }
 
 
+// View note names (in rectangles)
+void qtractorMidiEditorForm::viewNoteNames ( bool bOn )
+{
+	m_pMidiEditor->setNoteNames(bOn);
+	m_pMidiEditor->updateContents();
+}
+
+
 // View duration as widths of notes
 void qtractorMidiEditorForm::viewNoteDuration ( bool bOn )
 {
