@@ -1,7 +1,7 @@
 // qtractorSessionForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2020, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -41,9 +41,8 @@
 // qtractorSessionForm -- UI wrapper form.
 
 // Constructor.
-qtractorSessionForm::qtractorSessionForm (
-	QWidget *pParent, Qt::WindowFlags wflags )
-	: QDialog(pParent, wflags)
+qtractorSessionForm::qtractorSessionForm ( QWidget *pParent )
+	: QDialog(pParent)
 {
 	// Setup UI struct...
 	m_ui.setupUi(this);
@@ -142,6 +141,17 @@ void qtractorSessionForm::setSession ( qtractorSession *pSession )
 	// Session properties cloning...
 	m_props = pSession->properties();
 
+	// HACK: Fix for an initial session directory proposal...
+	if (m_props.sessionName.isEmpty()) {
+		QDir dir(m_props.sessionDir);
+		QStringList filters;
+		filters << dir.dirName() + '*';
+		if (!dir.entryList(filters, QDir::Files).isEmpty()) {
+			const QFileInfo info(dir.absolutePath());
+			m_props.sessionDir = info.absolutePath();
+		}
+	}
+
 	// Initialize dialog widgets...
 	m_ui.SessionNameLineEdit->setText(m_props.sessionName);
 	m_ui.SessionDirComboBox->setEditText(m_props.sessionDir);
